function [ y ] = FI_FIR_direct( b, x )
%FI_FIR_DIRECT Function that implements fixed_point direct realizaton of FIR filter
    
    % Sirine izlaznog signala se podesavaju prema sirinama ulaznog signala
    % i prema sirinama koeficijenata (moze i drugacije, ovde je tako usvojeno)
    fi_params = struct( 'OUT_SIGNAL_BITLENGTH', x.WordLength + b.WordLength, ...
                        'OUT_SIGNAL_FRAC', x.FractionLength + b.FractionLength);

    FixedPointAttributes = fimath( 'RoundingMethod', 'Floor', 'OverflowAction', 'Wrap', ...
        'ProductMode', 'SpecifyPrecision', 'ProductWordLength', fi_params.OUT_SIGNAL_BITLENGTH , 'ProductFractionLength', fi_params.OUT_SIGNAL_FRAC, ...
        'SumMode', 'SpecifyPrecision', 'SumWordLength', fi_params.OUT_SIGNAL_BITLENGTH, 'SumFractionLength', fi_params.OUT_SIGNAL_FRAC ) ;
    
    delay_line = fi( zeros(length(b),1), x.Signed, x.WordLength, x.FractionLength, FixedPointAttributes); 
    y = fi(zeros(1,length(x)), true, fi_params.OUT_SIGNAL_BITLENGTH, fi_params.OUT_SIGNAL_FRAC, FixedPointAttributes);
    
    for k = 1:length(x)
        delay_line = [x(k); delay_line(1:end-1)];
        y(k) = b*delay_line; 
    end
    % Obratiti paznju da matricno mnozenje radi sumiranje na nama nepoznat
    % nacin. Ako zelimo da imamo bas model hardvera, ovde mora da se
    % raspise petlja u kojoj se sabira element po element onako kako se to
    % radi u hardveru.
end

