clear all
close all
clc

%% Impulsni odziv filtra
b=[-0.0136 -0.0139 0.0254 0.0523 -0.0124 -0.0880 0.0252 0.3169 ...
0.4807 0.3169 0.0252 -0.0880 -0.0124 0.0523 0.0254 -0.0139 -0.0136] ;

figure
stem(0:length(b)-1, b);
xlabel('\it n');
ylabel('\it h \rm[\itn\rm]');

%% Definisanje ulaznog signala, zbir dve sinusoide
F1 =0.1; 
F2 =0.43; 
N = 256 ;
n = 0:N-1; 
x = sin(2*pi*F1*n) + sin(2*pi*F2*n);
x = x/(max([abs(min(x)) abs(max(x))])); % normalizacija 

%% Filtriranje i poredjenje sa MATLAB-ovom filter funkcijom
y = FIR_direct(b,x);
y1 = filter(b,1,x);
figure
subplot(411);
stem(n,x);
xlabel('\it n');
ylabel('\it x \rm[\itn\rm]');

subplot(412);
stem(n,y);
xlabel('\it n');
ylabel('\it y \rm[\itn\rm]');

subplot(413);
stem(n,y1);
xlabel('\it n');
ylabel('\it y\rm_1 [\itn\rm]');

subplot(414);
stem(n,y - y1);
xlabel('\it n');
ylabel('\it y - y\rm_1 [\itn\rm]');


%% Konverzija u fixed point i definisanje ponasanja sabiranja i mnozenja
% Podrazumevano je da se radi zaokruzivanje i da se radi zasicenje u
% slucaju prekoracenja
FixedPointAttributes=fimath ( 'RoundingMethod', 'Floor', 'OverflowAction', 'Saturate', ...
        'ProductMode', 'SpecifyPrecision', 'ProductWordLength', 32, 'ProductFractionLength', 30, ...
        'SumMode', 'SpecifyPrecision', 'SumWordLength', 32, 'SumFractionLength', 30 ) ;
fi_params = struct('FILTER_COEFITIENTS_BITLENGTH',  16, 'FILTER_COEFITIENTS_FRAC', 15, ...
                   'SIGNAL_BITLENGTH',              16, 'SIGNAL_FRAC',             15);

FI_b = fi ( b , true , fi_params.FILTER_COEFITIENTS_BITLENGTH , fi_params.FILTER_COEFITIENTS_FRAC, FixedPointAttributes);
FI_x = fi (x , true , fi_params.SIGNAL_BITLENGTH , fi_params.SIGNAL_FRAC, FixedPointAttributes);

% Zasicenje smo radili samo ulaznom signalu i koeficijentima, u filtru
% zasicenje zahteva dodatni hardver, pa nakon zaokruzivanja, vracamo 'Wrap'
FixedPointAttributes.OverflowAction = 'Wrap';
FI_b = fi ( FI_b , true , fi_params.FILTER_COEFITIENTS_BITLENGTH , fi_params.FILTER_COEFITIENTS_FRAC, FixedPointAttributes );
FI_x = fi ( FI_x , true , fi_params.SIGNAL_BITLENGTH , fi_params.SIGNAL_FRAC, FixedPointAttributes );


ex = x-double(FI_x);
SQNR_x = snr(x, ex)

y_fixed_point = FI_FIR_direct(FI_b,FI_x);

ey = y-double(y_fixed_point);
SQNR_y = snr(y,ey)

%% Plot rezultata
figure
subplot(411);
plot(n,x);
title('Ulazni signal');

subplot(412);
plot(n,y);
title('Izlazni signal sa double preciznoscu');

subplot(413);
plot(n,y_fixed_point);
title('Izlazni signal sa fixed point preciznoscu');

subplot(414);
plot(n,ey);
title('Razlika izlaznih signala sa double i sa fixed point preciznoscu');

%% Karakteristike filtara
[H,w]=freqz(b,1,1024);Ha=abs(H);
[H_fp,w]=freqz(double(fi(b, true, 8, 7)),1,1024);Ha_fp=abs(H_fp);
figure
plot(w,Ha,'LineWidth',2),title('Amplitudska karakteristika FIR filtra - linearna razmera'),grid on, hold on,
plot(w,Ha_fp,'r','LineWidth',2),
xlabel('w');
ylabel('|H(e^(jw))|');

%% Upis ulaznih odbiraka u fajl (integer)
x_integer = double(FI_x)*(2^fi_params.SIGNAL_FRAC);
f_ID = fopen('input_signal.txt','w');
for i = 1:length(x_integer)
    fprintf(f_ID,'%d\n',int32(x_integer(i)));
end
fclose(f_ID);

%% Koeficijenti filtra skalirani za VHDL
b_integer = double(FI_b)*(2^fi_params.FILTER_COEFITIENTS_FRAC);
for i = 1:length(b_integer)
    %fprintf('%X\n',int32(b_integer(i)));
    fprintf('x"%X",\n', typecast(int8(b_integer(i)),'uint8'));
end

